@@ -2,8 +2,6 @@
 # KiCad python module for interpreting generic netlists which can be used
 # to generate Bills of materials, etc.
 #
-# Remember these files use UTF8 encoding
-#
 # No string formatting is used on purpose as the only string formatting that
 # is current compatible with python 2.4+ to 3.0+ is the '%' method, and that
 # is due to be deprecated in 3.0+ soon
@@ -11,8 +9,10 @@
 
 """
     @package
-    Helper module for interpreting generic netlist and build custom
-    bom generators or netlists in foreign format
+    Generate a HTML BOM list.
+    Components are sorted and grouped by value
+    Fields are (if exist)
+    Ref, Quantity, Value, Part, Datasheet, Description, Vendor
 """
 
 
@@ -20,7 +20,7 @@ from __future__ import print_function
 import sys
 import xml.sax as sax
 import re
-import string
+import pdb
 
 #-----<Configure>----------------------------------------------------------------
 
@@ -35,7 +35,7 @@ excluded_fields = [
     ]
 
 
-# You may exclude components from the BOM by either:
+# You may exlude components from the BOM by either:
 #
 # 1) adding a custom field named "Installed" to your components and filling it
 # with a value of "NU" (Normally Uninstalled).
@@ -47,17 +47,17 @@ excluded_fields = [
 # regular expressions which match component 'Reference' fields of components that
 # are to be excluded from the BOM.
 excluded_references = [
-    # 'TP[0-9]+'              # all test points
+    'TP[0-9]+'              # all test points
     ]
 
 
 # regular expressions which match component 'Value' fields of components that
 # are to be excluded from the BOM.
 excluded_values = [
-    # 'MOUNTHOLE',
-    # 'SCOPETEST',
-    # 'MOUNT_HOLE',
-    # 'SOLDER_BRIDGE.*'
+    'MOUNTHOLE',
+    'SCOPETEST',
+    'MOUNT_HOLE',
+    'SOLDER_BRIDGE.*'
     ]
 
 
@@ -72,7 +72,7 @@ excluded_footprints = [
 
 class xmlElement():
     """xml element which can represent all nodes of the netlist tree.  It can be
-    used to easily generate various output formats by propagating format
+    used to easily generate various output formats by propogating format
     requests to children recursively.
     """
     def __init__(self, name, parent=None):
@@ -165,7 +165,6 @@ class xmlElement():
 
     def addAttribute(self, attr, value):
         """Add an attribute to this element"""
-        if type(value) != str: value = value.encode('utf-8')
         self.attributes[attr] = value
 
     def setAttribute(self, attr, value):
@@ -221,31 +220,20 @@ class xmlElement():
                 try:
                     if attrmatch != "":
                         if self.attributes[attribute] == attrmatch:
-                            ret = self.chars
-                            if type(ret) != str: ret = ret.encode('utf-8')
-                            return ret
+                            return self.chars
                     else:
-                        ret = self.attributes[attribute]
-                        if type(ret) != str: ret = ret.encode('utf-8')
-                        return ret
+                        return self.attributes[attribute]
                 except AttributeError:
-                    ret = ""
-                    if type(ret) != str: ret = ret.encode('utf-8')
-                    return ret
+                    return ""
             else:
-                ret = self.chars
-                if type(ret) != str: ret = ret.encode('utf-8')
-                return ret
+                return self.chars
 
         for child in self.children:
             ret = child.get(elemName, attribute, attrmatch)
             if ret != "":
-                if type(ret) != str: ret = ret.encode('utf-8')
                 return ret
 
-        ret = ""
-        if type(ret) != str: ret = ret.encode('utf-8')
-        return ret
+        return ""
 
 
 
@@ -285,16 +273,6 @@ class libpart():
                 fieldNames.append( f.get('field','name') )
         return fieldNames
 
-    def getPinList(self):
-        """Return a list of pins in play for this libpart.
-        """
-        pinList = []
-        pins = self.element.getChild('pins')
-        if pins:
-            for f in pins.getChildren():
-                pinList.append( f )
-        return pinList
-
     def getDatasheet(self):
         return self.getField("Datasheet")
 
@@ -328,26 +306,12 @@ class comp():
         self.grouped = False
 
     def __eq__(self, other):
-        """ Equivalency operator, remember this can be easily overloaded
-            2 components are equivalent ( i.e. can be grouped
-            if they have same value and same footprint and are both set to be populated
-
-            Override the component equivalence operator must be done before
-            loading the netlist, otherwise all components will have the original
-            equivalency operator.
-
-            You have to define a comparison module (for instance named myEqu)
-            and add the line;
-                kicad_netlist_reader.comp.__eq__ = myEqu
-            in your bom generator script before calling the netliste reader by something like:
-                net = kicad_netlist_reader.netlist(sys.argv[1])
-        """
+        """Equlivalency operator, remember this can be easily overloaded"""
         result = False
         if self.getValue() == other.getValue():
-            if self.getFootprint() == other.getFootprint():
-                if self.getRef().rstrip(string.digits) == other.getRef().rstrip(string.digits):
-                    if self.getDNP() == other.getDNP():
-                        result = True
+            if self.getLibName() == other.getLibName():
+                if self.getPartName() == other.getPartName():
+                    result = True
         return result
 
     def setLibPart(self, part):
@@ -371,21 +335,20 @@ class comp():
     def getValue(self):
         return self.element.get("value")
 
-    def getField(self, name, aLibraryToo = True):
-        """
-        Return the value of a field named name. The component is first
+    def getField(self, name, libraryToo=True):
+        """Return the value of a field named name. The component is first
         checked for the field, and then the components library part is checked
         for the field. If the field doesn't exist in either, an empty string is
         returned
 
         Keywords:
         name -- The name of the field to return the value for
-        aLibraryToo --  look in the libpart's fields for the same name if not found
+        libraryToo --   look in the libpart's fields for the same name if not found
                         in component itself
         """
 
         field = self.element.get("field", "name", name)
-        if field == "" and aLibraryToo and self.libpart:
+        if field == "" and libraryToo:
             field = self.libpart.getField(name)
         return field
 
@@ -405,124 +368,23 @@ class comp():
     def getRef(self):
         return self.element.get("comp", "ref")
 
-    '''
-    Return true if the component has the DNP property set
-    '''
-    def getDNP(self):
-        for child in self.element.getChildren( "property"):
-            try:
-                if child.attributes['name'] == 'dnp':
-                    return True
-            except KeyError:
-                continue
-
-        return False
-
-    '''
-    Return 'DNP' if the component has the DNP property set
-    '''
-    def getDNPString(self):
-        if self.getDNP():
-            return 'DNP'
-
-        return ''
-
-    '''
-    Return true if the component has the exclude from BOM property set
-    '''
-    def getExcludeFromBOM(self):
-        for child in self.element.getChildren( "property"):
-            try:
-                if child.attributes['name'] == 'exclude_from_bom':
-                    return True
-            except KeyError:
-                continue
-
-        return False
-
-    '''
-    Return true if the component has the exclude from Board property set
-    '''
-    def getExcludeFromBoard(self):
-        for child in self.element.getChildren( "property"):
-            try:
-                if child.attributes['name'] == 'exclude_from_board':
-                    return True
-            except KeyError:
-                continue
-
-        return False
-
-    '''
-    return the footprint name. if empty and aLibraryToo = True, return the
-    footprint name from libary
-    '''
-    def getFootprint(self, aLibraryToo = True):
+    def getFootprint(self, libraryToo=True):
         ret = self.element.get("footprint")
-
-        if ret == "" and aLibraryToo and self.libpart:
+        if ret =="" and libraryToo:
             ret = self.libpart.getFootprint()
-
         return ret
 
-    '''
-    return the datasheet name. if empty and aLibraryToo = True, return the
-    datasheet name from libary
-    '''
-    def getDatasheet(self, aLibraryToo = True):
+    def getDatasheet(self, libraryToo=True):
         ret = self.element.get("datasheet")
-        if ret == "" and aLibraryToo and self.libpart:
+        if ret == '' and libraryToo:
             ret = self.libpart.getDatasheet()
         return ret
 
     def getTimestamp(self):
-        """
-        Kicad 5 uses tstamp keyword for time stamp (8 digits) as UUID
-        Kicad 6 uses tstamps keyword for UUID and a multi unit symbol has more than one UUID
-        (UUIDs are separated by spaces)
-        """
-        ret = self.element.get("tstamp")
-        if ret == "":
-            ret = self.element.get("tstamps")
-        return ret
+        return self.element.get("tstamp")
 
     def getDescription(self):
-        return self.element.get("libsource", "description")
-
-    '''
-    return the netname of the pin aPinNum in netlist aNetlist
-    if aSkipEmptyNet = True, net having only one pin will return a empty name
-    '''
-    def getPinNetname(self, aPinNum, aNetlist, aSkipEmptyNet):
-        ref = self.getRef()
-
-        for net in aNetlist.nets:
-            net_name = net.get( "net", "name" )
-
-            item_cnt = 1
-            netitems = net.children
-
-            for node in netitems:
-                curr_item_ref = node.get( "node", "ref" )
-
-                if curr_item_ref == ref:
-                    curr_pin = node.get( "node", "pin" )
-
-                    if aPinNum == curr_pin:
-                        if aSkipEmptyNet:   #ensure at least 2 pins are in net
-                            pin_count = 0
-
-                            for curr_node in netitems:
-                                pin_count += 1
-
-                                if pin_count > 1:
-                                    return net_name
-
-                            return ""
-                        else:
-                            return net_name
-
-        return "?"
+        return self.libpart.getDescription()
 
 
 class netlist():
@@ -549,7 +411,7 @@ class netlist():
 
         self._curr_element = None
 
-        # component blacklist regexs, made from excluded_* above.
+        # component blacklist regexs, made from exluded_* above.
         self.excluded_references = []
         self.excluded_values = []
         self.excluded_footprints = []
@@ -563,7 +425,7 @@ class netlist():
 
     def addElement(self, name):
         """Add a new kicad generic element to the list"""
-        if self._curr_element is None:
+        if self._curr_element == None:
             self.tree = xmlElement(name)
             self._curr_element = self.tree
         else:
@@ -595,7 +457,7 @@ class netlist():
     def endDocument(self):
         """Called when the netlist document has been fully parsed"""
         # When the document is complete, the library parts must be linked to
-        # the components as they are separate in the tree so as not to
+        # the components as they are seperate in the tree so as not to
         # duplicate library part information for every component
         for c in self.components:
             for p in self.libparts:
@@ -635,10 +497,6 @@ class netlist():
         """Return the tool string which was used to create the netlist tree"""
         return self.design.get("tool")
 
-    def getNets(self):
-        """Return the nets """
-        return self.nets
-
     def gatherComponentFieldUnion(self, components=None):
         """Gather the complete 'set' of unique component fields, fields found in any component.
         """
@@ -682,12 +540,12 @@ class netlist():
 
         return ret       # this is a python 'set'
 
-    def getInterestingComponents(self, excludeBOM=False, excludeBoard=False, DNP=False):
+    def getInterestingComponents(self):
         """Return a subset of all components, those that should show up in the BOM.
         Omit those that should not, by consulting the blacklists:
         excluded_values, excluded_refs, and excluded_footprints, which hold one
-        or more regular expressions.  If any of the regular expressions match
-        the corresponding field's value in a component, then the component is excluded.
+        or more regular expressions.  If any of the the regular expressions match
+        the corresponding field's value in a component, then the component is exluded.
         """
 
         # pre-compile all the regex expressions:
@@ -715,38 +573,29 @@ class netlist():
                 for refs in self.excluded_references:
                     if refs.match(c.getRef()):
                         exclude = True
-                        break
+                        break;
             if not exclude:
                 for vals in self.excluded_values:
                     if vals.match(c.getValue()):
                         exclude = True
-                        break
+                        break;
             if not exclude:
                 for mods in self.excluded_footprints:
                     if mods.match(c.getFootprint()):
                         exclude = True
-                        break
+                        break;
 
-            if excludeBOM and c.getExcludeFromBOM():
-                exclude = True
-
-            if excludeBoard and c.getExcludeFromBoard():
-                exclude = True
-
-            if DNP and c.getDNP():
-                exclude = True
+            if not exclude:
+                # This is a fairly personal way to flag DNS (Do Not Stuff).  NU for
+                # me means Normally Uninstalled.  You can 'or in' another expression here.
+                if c.getField( "Installed" ) == 'NU':
+                    exclude = True
 
             if not exclude:
                 ret.append(c)
 
-        # The key to sort the components in the BOM
-        # This sorts using a natural sorting order (e.g. 100 after 99), and if it wasn't used
-        # the normal sort would place 100 before 99 since it only would look at the first digit.
-        def sortKey( str ):
-            return [ int(t) if t.isdigit() else t.lower()
-                    for t in re.split( '(\d+)', str ) ]
-
-        ret.sort(key=lambda g: sortKey(g.getRef()))
+        # Sort first by ref as this makes for easier to read BOM's
+        ret.sort(key=lambda g: g.getRef())
 
         return ret
 
@@ -785,19 +634,13 @@ class netlist():
                 # Add the new component group to the groups list
                 groups.append(newgroup)
 
-        # The key to sort the components in the BOM
-        # This sorts using a natural sorting order (e.g. 100 after 99), and if it wasn't used
-        # the normal sort would place 100 before 99 since it only would look at the first digit.
-        def sortKey( str ):
-            return [ int(t) if t.isdigit() else t.lower()
-                    for t in re.split( '(\d+)', str ) ]
-
+        # Each group is a list of components, we need to sort each list first
+        # to get them in order as this makes for easier to read BOM's
         for g in groups:
-            #g = sorted(g, key=lambda g: sortKey(g.getRef()))
-            g.sort(key=lambda g: sortKey(g.getRef()))
+            g = sorted(g, key=lambda g: g.getRef())
 
         # Finally, sort the groups to order the references alphabetically
-        groups.sort(key=lambda group: sortKey(group[0].getRef()))
+        groups = sorted(groups, key=lambda group: group[0].getRef())
 
         return groups
 
@@ -810,12 +653,7 @@ class netlist():
             ret = c.getField(field, False)
             if ret != '':
                 return ret
-
-        libpart = group[0].getLibPart()
-        if not libpart:
-            return ''
-
-        return libpart.getField(field)
+        return group[0].getLibPart().getField(field)
 
     def getGroupFootprint(self, group):
         """Return the whatever is known about the Footprint by consulting each
